import pygtk
# pygtk.require('2.0')
import gtk
import sys
import os
import gobject
from system import logger, workflow, dataset
import workflows 
       
"""Variable that keeps the path of the last open/save filedialog
with a successfull open/save, in other words, cancel was not
pressed in the dialog."""
_CURRENT_DIR = os.getcwd()

def get_yes_no(title, text, default_response=gtk.RESPONSE_NO,justify=gtk.JUSTIFY_LEFT):
    dlg = gtk.Dialog(title)
    dlg.show()

    text = gtk.Label(text)
    text.set_justify(justify)
    text.show()
    dlg.vbox.pack_start(text)

    dlg.add_button(gtk.STOCK_YES, gtk.RESPONSE_YES)
    dlg.add_button(gtk.STOCK_NO, gtk.RESPONSE_NO)
    dlg.set_default_response(default_response)
    dlg.connect("close", lambda *args : dlg.hide())
    response = dlg.run()

    retval = False

    if response == gtk.RESPONSE_YES:
        retval = True
    dlg.destroy()
    return retval

def get_confirm_quit(title, text):
    dlg = gtk.Dialog(title)
    dlg.show()

    text = gtk.Label(text)
    text.show()
    dlg.vbox.pack_start(text)

    dlg.add_button(gtk.STOCK_SAVE, gtk.RESPONSE_YES)
    dlg.add_button(gtk.STOCK_QUIT, gtk.RESPONSE_NO)
    dlg.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
    dlg.set_default_response(gtk.RESPONSE_CANCEL)
    dlg.connect("close", lambda *args : dlg.hide())
    response = dlg.run()

    retval = "CANCEL"

    if response == gtk.RESPONSE_YES:
        retval = "SAVE"
    if response == gtk.RESPONSE_NO:
        retval = "QUIT"
    dlg.destroy()
    return retval
    
def get_text(title, text):
    """Allow user to type in a string for text."""
    dlg = gtk.Dialog(title)
    dlg.show()

    text = gtk.Label(text)
    text.show()

    entry = gtk.Entry()
    entry.show()
    entry.set_activates_default(True)
    dlg.vbox.pack_start(text)
    dlg.vbox.pack_start(entry)

    dlg.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
    dlg.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
    dlg.set_default_response(gtk.RESPONSE_OK)
    dlg.connect("close", lambda *args : dlg.hide())
    response = dlg.run()

    retval = None

    if response == gtk.RESPONSE_OK:
        retval = entry.get_text()
    dlg.destroy()
    return retval

def make_filter(description, extension):
    f = gtk.FileFilter()
    f.set_name("%s (*.%s)" % (description, extension))
    # make "ext" into case insensitive pattern of form
    # "*.[eE][xX][tT]"
    f.add_pattern("*.[%s]" % "][".join([a+b for a, b in zip(extension.lower(),
                                                            extension.upper())]))
    return f

def get_save_filename(title, initial_name, filter_pair=None):
    """Open save filedialog to get filename as string.

    filter_pair is a pair or strings like ("My filetype", "ext").
    Returns None when users cancels.
    """  
    global _CURRENT_DIR
    chooser = gtk.FileChooserDialog(title=title, parent=None,
                                    action=gtk.FILE_CHOOSER_ACTION_SAVE,
                                    buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                             gtk.STOCK_SAVE, gtk.RESPONSE_OK))
    chooser.set_default_response(gtk.RESPONSE_OK)
    chooser.set_do_overwrite_confirmation(True)

    if filter_pair:
        description, extension = filter_pair
        chooser.add_filter(make_filter(description, extension))
        
    all_filter = gtk.FileFilter()
    all_filter.set_name("All Files")
    all_filter.add_pattern("*")
    chooser.add_filter(all_filter)
    path, filename = os.path.split(initial_name)
    if path:
        chooser.set_current_folder(path)
    else:
        chooser.set_current_folder(_CURRENT_DIR)
    chooser.set_current_name(filename)

    filename = None
    if chooser.run() == gtk.RESPONSE_OK:
        filename = chooser.get_filename()
        _CURRENT_DIR, t = os.path.split(filename)
    chooser.destroy()
    return filename


def get_open_filename(title, filter_pair=None, multiple=False):
    """Open filedialog to get filename as string.

    filter_pair is a pair or strings like ("My filetype", "ext").

    Returns None when users cancels for single file load, [] is
    returned when cancelling multiple file load.
    """  
    global _CURRENT_DIR
    chooser = gtk.FileChooserDialog(title=title, parent=None,
                                    action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                    buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                             gtk.STOCK_OPEN, gtk.RESPONSE_OK))
    chooser.set_default_response(gtk.RESPONSE_OK)
    chooser.set_select_multiple(multiple)

    if filter_pair:
        description, extension = filter_pair
        chooser.add_filter(make_filter(description, extension))
            
    all_filter = gtk.FileFilter()
    all_filter.set_name("All Files")
    all_filter.add_pattern("*")
    chooser.add_filter(all_filter)
    chooser.set_current_folder(_CURRENT_DIR)

    try:
        if chooser.run() == gtk.RESPONSE_OK:
            _CURRENT_DIR = chooser.get_current_folder()
            if multiple:
                return chooser.get_filenames()
            else:
                return chooser.get_filename()
        elif multiple:
            return []
        else:
            return None
            
    finally:
        chooser.destroy()
