import time
import Numeric


class OctaveFileWriter:
    supportedFileExtensions = ["oct"]
    fileDescription = "Octave"
    description = "GNU Octave file format."
    name = "OctaveFileWriter"


        
        
    def _write_data_object(self, name, data, file_):

        self._write_name(name, file_)

        if isinstance(data, list):
            self._write_numeric(Numeric.reshape(data, Numeric.shape(data)),
                                file_)
        elif isinstance(data, str):
            self._write_string(data, file_)
        elif isinstance(data, int) or isinstance(data, float):
            self._write_scalar(data, file_)
        elif isinstance(data, dict):
            self._write_struct(data, file_)
        else:
            # let's crash and burn if it's wrong!
            self._write_numeric(Numeric.array(data), file_)

    def _write_name(self, name, file_):
        file_.write('# name: %s\n' % name)        

    def _write_numeric(self, data, file_):
        if data.typecode() in Numeric.typecodes['Character']:
            # string array
            self._write_string(data, file_)
            return
          
        shape = Numeric.shape(data)

        if len(shape) == 1: # one-dimensional ; faking it into a two-dim one
            shape = (1, shape[0])
            data = [data]

        # start generating file
        file_.write('# type: matrix\n')

        if len(shape) < 3: # case regular two-dimensional
            rows, cols = shape[0], shape[1]
            file_.write('# rows: %s\n# columns: %s\n' % (rows, cols))

        else: # case; multidimensional array. Not supported in early octave versions
            file_.write('# ndims: ' + str(len(shape))+"\n" )
            file_.write(' '.join(map(str, shape)) +"\n")

            rows = shape[0]

            # data is transposed because that's how it octave reads them
            # (reshaping is just for ease of iterating over it
            data = Numeric.reshape(Numeric.transpose(data), (rows, -1))
                       
        for x in xrange(rows):
            file_.write(' %s\n' % ' '.join(map(str, data[x])))            

    def _write_string(self, data, file_):
        shape = Numeric.shape(data)
        if len(shape) == 1:
            data = [data]

        file_.write('# type: string\n')
        file_.write('# elements: %d\n' % len(data))
        for string in data:
            file_.write('# length: %d\n' % len(string))
            file_.write('%s\n' % ''.join(map(str, string)))

    def _write_scalar(self, data, file_):
        file_.write('# type: scalar\n')
        file_.write('%s\n' % data)

    def _write_struct(self, data, file_):
        file_.write('# type: struct\n')
        file_.write('# length: %d\n' % len(data))
        for name, data in data.items():
            self._write_data_object(name, data, file_)
            
    


    def write_variable_dict(self, vars_, filename):
        
        file_ = open(filename, 'wb')
        
        file_.write('# Created by %s %s\n' %("OctaveFileWriter",
                                             time.asctime(time.localtime(time.time()))))

        for name, value in vars_.items():
            self._write_data_object(name, value, file_)
        
        try:
            file_.close()
        except IOError, e:
            print 'Could not close %s' % filename

            
