HierarchicalClusterAnalysis <- function(X,metric="euclidean",method="divisive",rows="cluster rows",p=2){

#INPUT VARIABLES
# X      -            matrix with data used
# metric -            String with metric used
# method -            String with method used
# rows   -            String spescifying if you should cluster over
#                         the rows or the columns.
# p      -            Integer with the power of the minkowski metric.
# OUTPUT VARIABLES
# Clustering -        R-Object containing the following variables
#              height: a vector with the distances between merging clusters at the
#                      successive stages. 
#              ac: the agglomerative coefficient, measuring the clustering
#                      structure of the dataset.   
#              merge: an (n-1) by 2 matrix, where n is the number of observations. 
#                      Row i of 'merge' describes the merging of
#                      clusters at step i
#                      of the clustering.  If a number j in the row is
#                      negative,
#                      then the single observation |j| is merged at this stage.  If
#                      j is positive, then the merger is with the cluster formed at
#                      stage j of the algorithm.
#               order: a vector giving a permutation of the original observations to
#                      allow for plotting, in the sense that the branches of a
#                      clustering tree will not cross.
#  
#
#  
# Copyright(C), Endre Andersen and Jon Strom, The SciCraft Team, www.scicraft.org
#Last update: 26/3-06# $$$ ## This file is part of SciCraft.
# $$$ ##
# $$$ ## SciCraft is free software; you can redistribute it and/or modify it
# $$$ ## under the terms of the GNU General Public License as published by
# $$$ ## the Free Software Foundation; either version 2, or (at your option)
# $$$ ## any later version.
# $$$ ##
# $$$ ## SciCraft is distributed in the hope that it will be useful, but
# $$$ ## WITHOUT ANY WARANTY; without even the implied warranty of
# $$$ ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# $$$ ## General Public License for more details.
# $$$ ##
# $$$ ## You should have received a copy of the GNU General Public License
# $$$ ## along with R; see the file COPYING.  If not, write to the Free
# $$$ ## Software Foundation, 59 Temple Place - Suite 330, Boston, MA
# $$$ ## 02111-1307, USA.
# $$$ 
# $$$ ## -*- texinfo -*-
 
  source("ChemometricsFunctions/distances.r")
#metric "use input""euclidean", "maximum", "manhattan", "canberra","binary","minkowski"
#"pearson correlation","kendall correlation" "spearman correlation"
  library(cluster)
  X <- as.matrix(X)
  if(rows == "cluster rows"){
    rows <- "rows"}
  if(rows == "cluster columns"){
    rows = "columns"}
  
  
  if(metric=="use input"){
    if(nrow(X)!=ncol(X)) {
      stop("The input matrix is not a distance Matrix, which is
required if the metric used is 'use input' ")
    }
    else if(sum(X-t(X))!=0) {
      stop("The input matrix is not a distance Matrix, which is
         required if the metric used is 'use input' ")
    }
    else {
      DistMat <- X
    }
  }
  
  else{
    DistMat <- DistanceMatrix(X,metric = metric, p=p,rows=rows)
    DistMat <- DistMat$DistanceMatrix
  }
  
  if (method == "divisive"){
    Clustering <- diana(DistMat,diss=TRUE)}
  else{
    Clustering <- agnes(DistMat,method=method,diss=TRUE)}
  
  return(list(Clustering=Clustering)) 
  
}



#method: "divisive","average", "single" "complete" 
